@@ -7,8 +7,21 @@
 #include <QKeyEvent>
 #include <QDomDocument>
 #include <QFile>
+#include <QFileInfo>
 #include <QTextStream>
 
+// libmythbase headers
+#include "libmythbase/lcddevice.h"
+#include "libmythbase/mythcorecontext.h"
+#include "libmythbase/mythdate.h"
+#include "libmythbase/mythdb.h"
+#include "libmythbase/mythdirs.h"
+#include "libmythbase/mythlogging.h"
+#include "libmythbase/mythmedia.h"
+#include "libmythbase/mythplugin.h"
+#include "libmythbase/mythsystemlegacy.h"
+#include "libmythbase/mythversion.h"
+
 // libmythui headers
 #include "mythmainwindow.h"
 #include "mythdialogbox.h"
@@ -16,34 +29,7 @@
 #include "mythuitext.h"
 #include "mythuistatetype.h"
 #include "xmlparsebase.h"
-#include "mythsystemlegacy.h"
 #include "mythuihelper.h"
-#include "lcddevice.h"
-#include "mythcorecontext.h"
-
-// libmythbase headers
-#include "mythlogging.h"
-#include "mythdb.h"
-#include "mythdirs.h"
-#include "mythmedia.h"
-#include "mythversion.h"
-#include "mythdate.h"
-
-// libmythbase headers
-#include "mythplugin.h"
-
-MythThemedMenuState::MythThemedMenuState(MythScreenStack *parent,
-                                         const QString &name)
-    : MythScreenType(parent, name),
-      m_callback(NULL), m_callbackdata(NULL),
-      m_killable(false), m_loaded(false), m_titleState(NULL),
-      m_watermarkState(NULL), m_buttonList(NULL), m_descriptionText(NULL)
-{
-}
-
-MythThemedMenuState::~MythThemedMenuState()
-{
-}
 
 bool MythThemedMenuState::Create(void)
 {
@@ -68,7 +54,7 @@ bool MythThemedMenuState::Create(void)
 
 void MythThemedMenuState::CopyFrom(MythUIType *base)
 {
-    MythThemedMenuState *st = dynamic_cast<MythThemedMenuState *>(base);
+    auto *st = dynamic_cast<MythThemedMenuState *>(base);
     if (!st)
     {
         LOG(VB_GENERAL, LOG_INFO, "ERROR, bad parsing");
@@ -89,17 +75,21 @@ void MythThemedMenuState::CopyFrom(MythUIType *base)
 
 /** \brief Creates a themed menu.
  *
+ *  \note
+ *  The two ignore variables were used before commit c34f2a10 in
+ *  2008. Removing them would have a ripple effect into the callers.
+ *
+ *  \param cdir         The directory containing the theme. (Ignored)
  *  \param menufile     file name of menu definition file
  *  \param parent       the screen stack that owns this UI type
  *  \param name         the name of this UI type
+ *  \param allowreorder Allow reordering of the menu items. (Ignored)
  *  \param state        theme state associated with this menu
  */
-MythThemedMenu::MythThemedMenu(const QString &cdir, const QString &menufile,
+MythThemedMenu::MythThemedMenu(const QString &/*cdir*/, const QString &menufile,
                                MythScreenStack *parent, const QString &name,
-                               bool allowreorder, MythThemedMenuState *state)
-    : MythThemedMenuState(parent, name),
-      m_state(state), m_allocedstate(false), m_foundtheme(false),
-      m_ignorekeys(false), m_wantpop(false), m_menuPopup(NULL)
+                               bool /*allowreorder*/, MythThemedMenuState *state)
+    : MythThemedMenuState(parent, name), m_state(state)
 {
     if (!m_state)
     {
@@ -125,17 +115,19 @@ void MythThemedMenu::SetMenuTheme(const QString &menufile)
             m_foundtheme = true;
     }
     else
+    {
         m_foundtheme = true;
+    }
 
     if (!m_foundtheme)
         return;
 
     CopyFrom(m_state);
 
-    connect(m_buttonList, SIGNAL(itemSelected(MythUIButtonListItem*)),
-            SLOT(setButtonActive(MythUIButtonListItem*)));
-    connect(m_buttonList, SIGNAL(itemClicked(MythUIButtonListItem*)),
-            SLOT(buttonAction(MythUIButtonListItem*)));
+    connect(m_buttonList, &MythUIButtonList::itemSelected,
+            this, &MythThemedMenu::setButtonActive);
+    connect(m_buttonList, &MythUIButtonList::itemClicked,
+            this, qOverload<MythUIButtonListItem*>(&MythThemedMenu::buttonAction));
 
     if (!parseMenu(menufile))
         m_foundtheme = false;
@@ -149,11 +141,19 @@ MythThemedMenu::~MythThemedMenu(void)
 
 /// \brief Returns true iff a theme has been
 ///        found by a previous call to SetMenuTheme().
-bool MythThemedMenu::foundTheme(void)
+bool MythThemedMenu::foundTheme(void) const
 {
     return m_foundtheme;
 }
 
+/// \brief Get the themed menus callback function and data for that function
+void MythThemedMenu::getCallback(void (**lcallback)(void *, QString &),
+                                 void **data)
+{
+    *lcallback = m_state->m_callback;
+    *data = m_state->m_callbackdata;
+}
+
 /// \brief Set the themed menus callback function and data for that function
 void MythThemedMenu::setCallback(void (*lcallback)(void *, QString &),
                                  void *data)
@@ -174,7 +174,7 @@ QString MythThemedMenu::getSelection(void)
 
 void MythThemedMenu::setButtonActive(MythUIButtonListItem* item)
 {
-    ThemedButton button = item->GetData().value<ThemedButton>();
+    auto button = item->GetData().value<ThemedButton>();
     if (m_watermarkState)
     {
         if (!(m_watermarkState->DisplayState(button.type)))
@@ -212,18 +212,24 @@ bool MythThemedMenu::keyPressEvent(QKeyEvent *event)
 
     for (int i = 0; i < actions.size() && !handled; i++)
     {
-        QString action = actions[i];
+        const QString& action = actions[i];
         handled = true;
 
-        if (action == "ESCAPE" || action == "EXIT" || action == "EXITPROMPT")
+        if (action == "ESCAPE" ||
+            action == "EXIT" ||
+            action == "EXITPROMPT" ||
+            action == "STANDBYMODE")
         {
             bool    callbacks  = m_state->m_callback;
             bool    lastScreen = (GetMythMainWindow()->GetMainStack()
                                                      ->TotalScreens() == 1);
             QString menuaction = "UPMENU";
             QString selExit    = "EXITING_APP_PROMPT";
+
             if (action == "EXIT")
                 selExit = "EXITING_APP";
+            else if (action == "STANDBYMODE")
+                selExit = "STANDBY_MODE";
 
             if (!m_allocedstate)
                 handleAction(menuaction);
@@ -244,6 +250,7 @@ bool MythThemedMenu::keyPressEvent(QKeyEvent *event)
                 }
             }
             else if ((action == "EXIT" || action == "EXITPROMPT" ||
+                      action == "STANDBYMODE" ||
                       (action == "ESCAPE" &&
                        (QCoreApplication::applicationName() ==
                         MYTH_APPNAME_MYTHTV_SETUP))) && lastScreen)
@@ -266,7 +273,9 @@ bool MythThemedMenu::keyPressEvent(QKeyEvent *event)
             handleAction(action);
         }
         else
+        {
             handled = false;
+        }
     }
 
     if (!handled && MythScreenType::keyPressEvent(event))
@@ -275,7 +284,7 @@ bool MythThemedMenu::keyPressEvent(QKeyEvent *event)
     m_ignorekeys = false;
 
     if (m_wantpop)
-        m_ScreenStack->PopScreen();
+        m_screenStack->PopScreen();
 
     return handled;
 }
@@ -303,30 +312,36 @@ void MythThemedMenu::ShowMenu()
 
     // HACK Implement a better check for this
     if (QCoreApplication::applicationName() == MYTH_APPNAME_MYTHFRONTEND)
-        m_menuPopup->AddButton(tr("Enter standby mode"), QVariant("standby"));
+        m_menuPopup->AddButtonV(tr("Enter standby mode"), QVariant("standby"));
+
+    // don't show the exit application option if standby option is enabled
+    if (override_menu != 7)
+        m_menuPopup->AddButtonV(tr("Exit application"), QVariant("exit"));
+
     switch (override_menu)
     {
         case 2:
         case 4:
             // shutdown
-            m_menuPopup->AddButton(tr("Shutdown"), QVariant("shutdown"));
+            m_menuPopup->AddButtonV(tr("Shutdown"), QVariant("shutdown"));
             break;
         case 5:
             // reboot
-            m_menuPopup->AddButton(tr("Reboot"), QVariant("reboot"));
+            m_menuPopup->AddButtonV(tr("Reboot"), QVariant("reboot"));
             break;
         case 3:
         case 6:
             // both
-            m_menuPopup->AddButton(tr("Shutdown"), QVariant("shutdown"));
-            m_menuPopup->AddButton(tr("Reboot"), QVariant("reboot"));
+            m_menuPopup->AddButtonV(tr("Shutdown"), QVariant("shutdown"));
+            m_menuPopup->AddButtonV(tr("Reboot"), QVariant("reboot"));
             break;
         case 0:
+        case 7:
         default:
             break;
     }
 
-    m_menuPopup->AddButton(tr("About"), QVariant("about"));
+    m_menuPopup->AddButtonV(tr("About"), QVariant("about"));
 }
 
 void MythThemedMenu::aboutScreen()
@@ -342,9 +357,9 @@ void MythThemedMenu::aboutScreen()
     }
 
     QString label = tr("Revision: %1\n Branch: %2\n %3")
-                        .arg(MYTH_SOURCE_VERSION)
-                        .arg(MYTH_SOURCE_PATH)
-                        .arg(distro_line);
+                        .arg(GetMythSourceVersion(),
+                             GetMythSourcePath(),
+                             distro_line);
 
     MythScreenStack* mainStack = GetMythMainWindow()->GetMainStack();
     m_menuPopup = new MythDialogBox(label, mainStack, "version_dialog");
@@ -360,7 +375,7 @@ void MythThemedMenu::customEvent(QEvent *event)
 {
     if (event->type() == DialogCompletionEvent::kEventType)
     {
-        DialogCompletionEvent *dce = (DialogCompletionEvent*)(event);
+        auto *dce = (DialogCompletionEvent*)(event);
 
         QString resultid = dce->GetId();
         //int buttonnum = dce->GetResult();
@@ -389,25 +404,30 @@ void MythThemedMenu::customEvent(QEvent *event)
                 QString arg("standby_mode");
                 m_state->m_callback(m_state->m_callbackdata, arg);
             }
+            else if (action == "exit")
+            {
+                QString arg("exiting_app");
+                m_state->m_callback(m_state->m_callbackdata, arg);
+            }
         }
         else if (resultid == "password")
         {
             QString text = dce->GetResultText();
             MythUIButtonListItem *item = m_buttonList->GetItemCurrent();
-            ThemedButton button = item->GetData().value<ThemedButton>();
+            auto button = item->GetData().value<ThemedButton>();
             QString password = GetMythDB()->GetSetting(button.password);
             if (text == password)
             {
                 QString timestamp_setting = QString("%1Time").arg(button.password);
                 QDateTime curr_time = MythDate::current();
-                QString last_time_stamp = 
+                QString last_time_stamp =
                     MythDate::toString(curr_time, MythDate::kDatabase);
                 GetMythDB()->SaveSetting(timestamp_setting, last_time_stamp);
                 buttonAction(item, true);
             }
         }
 
-        m_menuPopup = NULL;
+        m_menuPopup = nullptr;
     }
 }
 
@@ -442,17 +462,13 @@ void MythThemedMenu::parseThemeButton(QDomElement &element)
                 if (text.isEmpty() &&
                     info.attribute("lang","").isEmpty())
                 {
-                    text = qApp->translate("ThemeUI",
-                                           parseText(info).toUtf8(), NULL,
-                                           QCoreApplication::UnicodeUTF8);
+                    text = QCoreApplication::translate("ThemeUI",
+                                           parseText(info).toUtf8() );
                 }
-                else if (info.attribute("lang","").toLower() ==
-                         gCoreContext->GetLanguageAndVariant())
-                {
-                    text = parseText(info);
-                }
-                else if (info.attribute("lang","").toLower() ==
-                         gCoreContext->GetLanguage())
+                else if ((info.attribute("lang","").toLower() ==
+                          gCoreContext->GetLanguageAndVariant()) ||
+                         (info.attribute("lang","").toLower() ==
+                          gCoreContext->GetLanguage()))
                 {
                     text = parseText(info);
                 }
@@ -462,17 +478,13 @@ void MythThemedMenu::parseThemeButton(QDomElement &element)
                 if (alttext.isEmpty() &&
                     info.attribute("lang","").isEmpty())
                 {
-                    alttext = qApp->translate("ThemeUI",
-                                              parseText(info).toUtf8(), NULL,
-                                              QCoreApplication::UnicodeUTF8);
+                    alttext = QCoreApplication::translate("ThemeUI",
+                                              parseText(info).toUtf8());
                 }
-                else if (info.attribute("lang","").toLower() ==
-                         gCoreContext->GetLanguageAndVariant())
-                {
-                    alttext = parseText(info);
-                }
-                else if (info.attribute("lang","").toLower() ==
-                         gCoreContext->GetLanguage())
+                else if ((info.attribute("lang","").toLower() ==
+                          gCoreContext->GetLanguageAndVariant()) ||
+                         (info.attribute("lang","").toLower() ==
+                          gCoreContext->GetLanguage()))
                 {
                     alttext = parseText(info);
                 }
@@ -485,9 +497,13 @@ void MythThemedMenu::parseThemeButton(QDomElement &element)
             {
                 addit = findDepends(getFirstText(info));
             }
+            else if (info.tagName() == "dependsexec")
+            {
+                addit = findDependsExec(getFirstText(info));
+            }
             else if (info.tagName() == "dependssetting")
             {
-                addit = GetMythDB()->GetNumSetting(getFirstText(info));
+                addit = GetMythDB()->GetBoolSetting(getFirstText(info));
             }
             else if (info.tagName() == "dependjumppoint")
             {
@@ -508,18 +524,13 @@ void MythThemedMenu::parseThemeButton(QDomElement &element)
                 if (description.isEmpty() &&
                     info.attribute("lang","").isEmpty())
                 {
-                    description = qApp->translate("ThemeUI",
-                                                  getFirstText(info).toUtf8(),
-                                                  NULL,
-                                                  QCoreApplication::UnicodeUTF8);
+                    description = QCoreApplication::translate("ThemeUI",
+                                                  getFirstText(info).toUtf8());
                 }
-                else if (info.attribute("lang","").toLower() ==
-                         gCoreContext->GetLanguageAndVariant())
-                {
-                    description = getFirstText(info);
-                }
-                else if (info.attribute("lang","").toLower() ==
-                         gCoreContext->GetLanguage())
+                else if ((info.attribute("lang","").toLower() ==
+                          gCoreContext->GetLanguageAndVariant()) ||
+                         (info.attribute("lang","").toLower() ==
+                          gCoreContext->GetLanguage()))
                 {
                     description = getFirstText(info);
                 }
@@ -584,6 +595,7 @@ bool MythThemedMenu::parseMenu(const QString &menuname)
         return false;
     }
 
+#if QT_VERSION < QT_VERSION_CHECK(6,5,0)
     QString errorMsg;
     int errorLine = 0;
     int errorColumn = 0;
@@ -600,6 +612,22 @@ bool MythThemedMenu::parseMenu(const QString &menuname)
                         .arg(menuname));
         return false;
     }
+#else
+    auto parseResult = doc.setContent(&f);
+    if (!parseResult)
+    {
+        LOG(VB_GENERAL, LOG_ERR,
+            QString("Error parsing: %1\nat line: %2  column: %3 msg: %4")
+            .arg(filename).arg(parseResult.errorLine)
+            .arg(parseResult.errorColumn).arg(parseResult.errorMessage));
+        f.close();
+
+        if (menuname != "mainmenu.xml")
+            ShowOkPopup(tr("The menu file %1 is incomplete.")
+                        .arg(menuname));
+        return false;
+    }
+#endif
 
     f.close();
 
@@ -653,10 +681,17 @@ bool MythThemedMenu::parseMenu(const QString &menuname)
  *         m_state and the type, text, alt-text and action provided in
  *         the parameters.
  *
- *  \param type    type of button to be created
- *  \param text    text to appear on the button
- *  \param alttext alternate text to appear when required
- *  \param action  actions to be associated with button
+ *  Buttons (or the action of a button) can be locked and requie a
+ *  password to be used.  This is indicated by setting the password
+ *  parameter.
+ *
+ *  \param type        type of button to be created
+ *  \param text        text to appear on the button
+ *  \param alttext     alternate text to appear when required
+ *  \param action      actions to be associated with button
+ *  \param description XXX
+ *  \param password    The name of the setting that stores the
+ *                     password.
  */
 void MythThemedMenu::addButton(const QString &type, const QString &text,
                                 const QString &alttext,
@@ -668,15 +703,15 @@ void MythThemedMenu::addButton(const QString &type, const QString &text,
     newbutton.type = type;
     newbutton.action = action;
     newbutton.text = text;
+    newbutton.alttext = alttext;
     newbutton.description = description;
     newbutton.password = password;
 
     if (m_watermarkState)
         m_watermarkState->EnsureStateLoaded(type);
 
-    MythUIButtonListItem *listbuttonitem =
-                                new MythUIButtonListItem(m_buttonList, text,
-                                                qVariantFromValue(newbutton));
+    auto *listbuttonitem = new MythUIButtonListItem(m_buttonList, text,
+                                                QVariant::fromValue(newbutton));
 
     listbuttonitem->DisplayState(type, "icon");
     listbuttonitem->SetText(description, "description");
@@ -684,20 +719,24 @@ void MythThemedMenu::addButton(const QString &type, const QString &text,
 
 void MythThemedMenu::buttonAction(MythUIButtonListItem *item, bool skipPass)
 {
-    ThemedButton button = item->GetData().value<ThemedButton>();
+    auto button = item->GetData().value<ThemedButton>();
 
     QString password;
     if (!skipPass)
         password = button.password;
 
-    QStringList::Iterator it = button.action.begin();
-    for (; it != button.action.end(); it++)
+    for (const auto & act : std::as_const(button.action))
     {
-        if (handleAction(*it, password))
+        if (handleAction(act, password))
             break;
     }
 }
 
+void MythThemedMenu::buttonAction(MythUIButtonListItem *item)
+{
+    buttonAction(item, false);
+}
+
 /** \brief Locates the appropriate menu file from which to parse the menu
  *
  *  \param menuname file name of the menu you want to find
@@ -709,50 +748,45 @@ QString MythThemedMenu::findMenuFile(const QString &menuname)
     QFile file(testdir);
     if (file.exists())
         return testdir;
-    else
-        LOG(VB_FILE, LOG_DEBUG, "No menu file " + testdir);
+    LOG(VB_FILE, LOG_DEBUG, "No menu file " + testdir);
 
     testdir = GetMythUI()->GetMenuThemeDir() + '/' + menuname;
     file.setFileName(testdir);
     if (file.exists())
         return testdir;
-    else
-        LOG(VB_FILE, LOG_DEBUG, "No menu file " + testdir);
+    LOG(VB_FILE, LOG_DEBUG, "No menu file " + testdir);
 
     testdir = GetMythUI()->GetThemeDir() + '/' + menuname;
     file.setFileName(testdir);
     if (file.exists())
         return testdir;
-    else
-        LOG(VB_FILE, LOG_DEBUG, "No menu file " + testdir);
+    LOG(VB_FILE, LOG_DEBUG, "No menu file " + testdir);
 
     testdir = GetShareDir() + menuname;
     file.setFileName(testdir);
     if (file.exists())
         return testdir;
-    else
-        LOG(VB_FILE, LOG_DEBUG, "No menu file " + testdir);
+    LOG(VB_FILE, LOG_DEBUG, "No menu file " + testdir);
 
     testdir = "../mythfrontend/" + menuname;
     file.setFileName(testdir);
     if (file.exists())
         return testdir;
-    else
-        LOG(VB_FILE, LOG_DEBUG, "No menu file " + testdir);
+    LOG(VB_FILE, LOG_DEBUG, "No menu file " + testdir);
 
     testdir = GetShareDir() + "themes/defaultmenu/" + menuname;
     file.setFileName(testdir);
     if (file.exists())
         return testdir;
-    else
-        LOG(VB_FILE, LOG_DEBUG, "No menu file " + testdir);
+    LOG(VB_FILE, LOG_DEBUG, "No menu file " + testdir);
 
-    return QString();
+    return {};
 }
 
 /** \brief Handle a MythTV action for the Menus.
  *
  *  \param action single action to be handled
+ *  \param password  password setting to be checked
  *  \return true if the action is not to EXEC another program
  */
 bool MythThemedMenu::handleAction(const QString &action, const QString &password)
@@ -770,7 +804,7 @@ bool MythThemedMenu::handleAction(const QString &action, const QString &password
 
         return false;
     }
-    else if (action.startsWith("EXECTV "))
+    if (action.startsWith("EXECTV "))
     {
         QString rest = action.right(action.length() - 7).trimmed();
         if (cbs && cbs->exec_program_tv)
@@ -782,8 +816,7 @@ bool MythThemedMenu::handleAction(const QString &action, const QString &password
 
         MythScreenStack *stack = GetScreenStack();
 
-        MythThemedMenu *newmenu = new MythThemedMenu("", menu, stack, menu,
-                                                     false, m_state);
+        auto *newmenu = new MythThemedMenu("", menu, stack, menu, false, m_state);
         if (newmenu->foundTheme())
             stack->AddScreen(newmenu);
         else
@@ -845,17 +878,16 @@ bool MythThemedMenu::handleAction(const QString &action, const QString &password
 bool MythThemedMenu::findDepends(const QString &fileList)
 {
     QStringList files = fileList.split(" ");
-    QString filename;
     MythPluginManager *pluginManager = gCoreContext->GetPluginManager();
 
-    for (QStringList::Iterator it = files.begin(); it != files.end(); ++it )
+    for (const auto & file : std::as_const(files))
     {
-        QString filename = findMenuFile(*it);
+        QString filename = findMenuFile(file);
         if (!filename.isEmpty() && filename.endsWith(".xml"))
             return true;
 
         // Has plugin by this name been successfully loaded
-        MythPlugin *plugin = pluginManager->GetPlugin(*it);
+        MythPlugin *plugin = pluginManager->GetPlugin(file);
         if (plugin)
             return true;
     }
@@ -863,12 +895,17 @@ bool MythThemedMenu::findDepends(const QString &fileList)
     return false;
 }
 
+bool MythThemedMenu::findDependsExec(const QString &filename)
+{
+    QFileInfo filename_info(filename);
+
+    return filename_info.exists() && filename_info.isFile() && filename_info.isExecutable();
+}
+
 /** \brief Queries the user for a password to enter a part of MythTV
  *         restricted by a password.
  *
- *  \param timestamp_setting time settings to be checked
  *  \param password_setting  password to be checked
- *  \param text              the message text to be displayed
  *  \return true if password checks out or is not needed.
  */
 bool MythThemedMenu::checkPinCode(const QString &password_setting)
@@ -905,8 +942,7 @@ bool MythThemedMenu::checkPinCode(const QString &password_setting)
 
     QString text = tr("Enter password:");
     MythScreenStack *popupStack = GetMythMainWindow()->GetStack("popup stack");
-    MythTextInputDialog *dialog =
-            new MythTextInputDialog(popupStack, text, FilterNone, true);
+    auto *dialog = new MythTextInputDialog(popupStack, text, FilterNone, true);
 
     if (dialog->Create())
     {
@@ -914,7 +950,9 @@ bool MythThemedMenu::checkPinCode(const QString &password_setting)
         popupStack->AddScreen(dialog);
     }
     else
+    {
         delete dialog;
+    }
 
     return false;
 }
@@ -942,10 +980,8 @@ void MythThemedMenu::mediaEvent(MythMediaEvent* event)
     switch (type)
     {
         case MEDIATYPE_DVD :
-            // DVD Available
-            break;
         case MEDIATYPE_BD :
-            // Blu-ray Available
+            // DVD or Blu-ray Available
             break;
         default :
             return;
